@@ -1,4 +1,5 @@
-/* AnjutaPluginDescription - Plugin meta data
+/*
+ * AnjutaPluginDescription - Plugin meta data
  * anjuta-plugin-description.c Copyright (C) 2002 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -13,8 +14,18 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
+
+/**
+ * SECTION:anjuta-plugin-description
+ * @title: AnjutaPluginDescription
+ * @short_description: Plugin description from .plugin file
+ * @see_also: #AnjutaPlugin, #AnjutaPluginHandle
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-plugin-description.h
+ * 
  */
 
 #include <string.h>
@@ -432,7 +443,7 @@ parse_key_value (AnjutaPluginDescriptionParser *parser, GError **error)
   gchar *key_end;
   gchar *key;
   gchar *locale_start = NULL;
-  gchar *locale_end;
+  gchar *locale_end = NULL;
   gchar *value_start;
   gchar *value;
   gchar *p;
@@ -516,7 +527,7 @@ parse_key_value (AnjutaPluginDescriptionParser *parser, GError **error)
     line->locale = g_strndup (locale_start, locale_end - locale_start);
   line->value = value;
   
-  parser->line = (line_end) ? line_end + 1 : NULL;
+  parser->line = (*line_end) ? line_end + 1 : NULL;
   parser->line_nr++;
   
   return TRUE;
@@ -914,7 +925,7 @@ anjuta_plugin_description_get_string (AnjutaPluginDescription   *df,
  * @keyname: Key name.
  * @val: Pointer to value to store retured value.
  * 
- * Returns the value of key as integer in the given section in current locale.
+ * Returns the value of key as integer in the given section.
  *
  * Return value: TRUE if sucessful, otherwise FALSE.
  */
@@ -940,3 +951,50 @@ anjuta_plugin_description_get_integer (AnjutaPluginDescription   *df,
   return TRUE;
   
 }
+
+/**
+ * anjuta_plugin_description_get_boolean:
+ * @df: an #AnjutaPluginDescription object.
+ * @section: Section name.
+ * @keyname: Key name.
+ * @val: Pointer to value to store retured value.
+ * 
+ * Returns the value of key as boolean in the given section.
+ *
+ * Return value: TRUE if sucessful, otherwise FALSE.
+ */
+gboolean
+anjuta_plugin_description_get_boolean (AnjutaPluginDescription   *df,
+			      const char       *section,
+			      const char       *keyname,
+			      gboolean         *val)
+{
+  gboolean res;
+  char *str;
+  
+  *val = 0;
+
+  res = anjuta_plugin_description_get_raw (df, section, keyname, NULL, &str);
+  if (!res)
+    return FALSE;
+
+  if ((g_ascii_strcasecmp (str, "yes") == 0) ||
+      (g_ascii_strcasecmp (str, "true") == 0))
+  {
+    *val = TRUE;
+  }
+  else if ((g_ascii_strcasecmp (str, "no") == 0) ||
+	          (g_ascii_strcasecmp (str, "false") == 0))	  
+
+  {
+    *val = FALSE;
+  }
+  else
+  {
+     res = FALSE;
+  }
+
+  g_free (str);
+
+  return res;
+}
