@@ -30,13 +30,13 @@
 /*#define DEBUG*/
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-message-manager.h>
-#include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-register.h>
+#include <libanjuta/interfaces/ianjuta-debugger-memory.h>
+#include <libanjuta/interfaces/ianjuta-debugger-instruction.h>
 #include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
 #include <libanjuta/interfaces/ianjuta-debugger-variable.h>
 
 
-#include <stdarg.h>
-
 /* Contants defintion
  *---------------------------------------------------------------------------*/
 
@@ -156,7 +156,7 @@ dma_debugger_queue_clear (DmaDebuggerQueue *self)
 }
 		
 static void
-dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState state, GError* err)
+dma_queue_emit_debugger_state_change (DmaDebuggerQueue *self, IAnjutaDebuggerState state, GError* err)
 {
 	enum
 	{
@@ -170,102 +170,56 @@ dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState stat
 		PROGRAM_RUNNING_SIGNAL,
 		PROGRAM_STOPPED_SIGNAL
 	} signal = NO_SIGNAL;
-	gboolean emit_program_started = FALSE;
 	
-	DEBUG_PRINT("update debugger state new %d old %d", state, self->debugger_state);
+	DEBUG_PRINT("change debugger state new %d old %d", state, self->debugger_state);
 
-	/* Add missing state if useful */
 	switch (state)
 	{
+	case IANJUTA_DEBUGGER_BUSY:
+		/* Debugger is busy, nothing to do */
+		g_return_if_reached();
+		return;
 	case IANJUTA_DEBUGGER_STOPPED:
-		if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
-			(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
-		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
-		}
-		else if	(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_LOADED)
-		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
-		}
+		self->stop_on_sharedlib = FALSE;
+		signal = DEBUGGER_STOPPED_SIGNAL;
+		self->debugger_state = state;
 		break;
 	case IANJUTA_DEBUGGER_STARTED:
-		if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
-			(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
-		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
-		}
+		self->stop_on_sharedlib = FALSE;
+		signal = self->debugger_state < IANJUTA_DEBUGGER_STARTED ? DEBUGGER_STARTED_SIGNAL : PROGRAM_UNLOADED_SIGNAL;			
+		self->debugger_state = state;
 		break;
 	case IANJUTA_DEBUGGER_PROGRAM_LOADED:
-		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
-		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
-		}
+		self->stop_on_sharedlib = FALSE;
+		signal = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_LOADED ? PROGRAM_LOADED_SIGNAL : PROGRAM_EXITED_SIGNAL;			
+		self->debugger_state = state;
 		break;
 	case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
-		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+		if (self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED)
 		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
+			signal = PROGRAM_STARTED_SIGNAL;
+			/* Emit a debugger stopped after program started */
+			self->debugger_state = IANJUTA_DEBUGGER_PROGRAM_RUNNING;
+			break;
 		}
-		else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
+		if (!self->stop_on_sharedlib)
 		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
+			signal = PROGRAM_STOPPED_SIGNAL;			
 		}
+		self->debugger_state = state;
 		break;
 	case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
-		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+		self->stop_on_sharedlib = FALSE;
+		if (self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED)
 		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
-		}
-		else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
-		{
-			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
+			signal = PROGRAM_STARTED_SIGNAL;
+			/* Emit a debugger stopped after program started */
+			self->debugger_state = IANJUTA_DEBUGGER_PROGRAM_STOPPED;
+			break;
 		}
+		signal = PROGRAM_RUNNING_SIGNAL;
+		self->debugger_state = state;
 		break;
-	case IANJUTA_DEBUGGER_BUSY:
-		break;
-	}	
-
-	if (self->debugger_state != state)
-	{
-		switch (state)
-		{
-		case IANJUTA_DEBUGGER_BUSY:
-			/* Debugger is busy, nothing to do */
-			return;
-		case IANJUTA_DEBUGGER_STOPPED:
-			self->stop_on_sharedlib = FALSE;
-			signal = DEBUGGER_STOPPED_SIGNAL;
-			self->debugger_state = state;
-			break;
-		case IANJUTA_DEBUGGER_STARTED:
-			self->stop_on_sharedlib = FALSE;
-			signal = self->debugger_state < IANJUTA_DEBUGGER_STARTED ? DEBUGGER_STARTED_SIGNAL : PROGRAM_UNLOADED_SIGNAL;			
-			self->debugger_state = state;
-			break;
-		case IANJUTA_DEBUGGER_PROGRAM_LOADED:
-			self->stop_on_sharedlib = FALSE;
-			signal = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_LOADED ? PROGRAM_LOADED_SIGNAL : PROGRAM_EXITED_SIGNAL;			
-			self->debugger_state = state;
-			break;
-		case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
-			if (!self->stop_on_sharedlib)
-			{
-				emit_program_started = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED;
-				signal = PROGRAM_STOPPED_SIGNAL;			
-			}
-			else if (self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED)
-			{
-				signal = PROGRAM_STARTED_SIGNAL;
-			}
-			self->debugger_state = state;
-			break;
-		case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
-			emit_program_started = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED;
-			self->stop_on_sharedlib = FALSE;
-			signal = PROGRAM_RUNNING_SIGNAL;			
-			self->debugger_state = state;
-			break;
-		}
 	}
 
 	self->prepend_command++;
@@ -276,52 +230,104 @@ dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState stat
 		/* Do nothing */
 		break;
 	case DEBUGGER_STOPPED_SIGNAL:
-		DEBUG_PRINT("** emit debugger-stopped **");
+		DEBUG_PRINT("%s", "** emit debugger-stopped **");
 		g_signal_emit_by_name (self->plugin, "debugger-stopped", err);
 		break;
 	case DEBUGGER_STARTED_SIGNAL:
-		DEBUG_PRINT("** emit debugger-started **");
+		DEBUG_PRINT("%s", "** emit debugger-started **");
 		g_signal_emit_by_name (self->plugin, "debugger-started");
 		break;
 	case PROGRAM_LOADED_SIGNAL:
-		DEBUG_PRINT("** emit program-loaded **");
+		DEBUG_PRINT("%s", "** emit program-loaded **");
 		g_signal_emit_by_name (self->plugin, "program-loaded");
 		break;
 	case PROGRAM_UNLOADED_SIGNAL:
-		DEBUG_PRINT("** emit program-unloaded **");
+		DEBUG_PRINT("%s", "** emit program-unloaded **");
 		g_signal_emit_by_name (self->plugin, "program-unloaded");
 		break;
 	case PROGRAM_STARTED_SIGNAL:
-		DEBUG_PRINT("** emit program-started **");
+		DEBUG_PRINT("%s", "** emit program-started **");
 		g_signal_emit_by_name (self->plugin, "program-started");
 		break;
 	case PROGRAM_EXITED_SIGNAL:
-		DEBUG_PRINT("** emit program-exited **");
+		DEBUG_PRINT("%s", "** emit program-exited **");
 		g_signal_emit_by_name (self->plugin, "program-exited");
 		break;
 	case PROGRAM_STOPPED_SIGNAL:
-		DEBUG_PRINT("** emit program-exited **");
-		if (emit_program_started)
-		{	
-			DEBUG_PRINT("** emit program-started **");
-			g_signal_emit_by_name (self->plugin, "program-started");
-		}
-		DEBUG_PRINT("** emit program-stopped **");
+		DEBUG_PRINT("%s", "** emit program-stopped **");
 		g_signal_emit_by_name (self->plugin, "program-stopped");
 		break;
 	case PROGRAM_RUNNING_SIGNAL:
-		if (emit_program_started)
-		{	
-			DEBUG_PRINT("** emit program-started **");
-			g_signal_emit_by_name (self->plugin, "program-started");
-		}
-		DEBUG_PRINT("** emit program-running **");
+		DEBUG_PRINT("%s", "** emit program-running **");
 		g_signal_emit_by_name (self->plugin, "program-running");
 		break;
 	}
 	self->prepend_command--;
 }
 
+static void
+dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState state, GError* err)
+{
+	DEBUG_PRINT("update debugger state new %d old %d", state, self->debugger_state);
+
+	/* Add missing states if useful */
+	for(;self->debugger_state != state;)
+	{
+		IAnjutaDebuggerState next_state = state;
+		
+		switch (state)
+		{
+		case IANJUTA_DEBUGGER_STOPPED:
+			if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
+				(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
+			{
+				next_state = IANJUTA_DEBUGGER_PROGRAM_LOADED;
+			}
+			else if	(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_LOADED)
+			{
+				next_state = IANJUTA_DEBUGGER_STARTED;
+			}
+			break;
+		case IANJUTA_DEBUGGER_STARTED:
+			if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
+				(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
+			{
+				next_state = IANJUTA_DEBUGGER_PROGRAM_LOADED;
+			}
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_LOADED:
+			if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+			{
+				next_state = IANJUTA_DEBUGGER_STARTED;
+			}
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
+			if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+			{
+				next_state = IANJUTA_DEBUGGER_STARTED;
+			}
+			else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
+			{
+				next_state = IANJUTA_DEBUGGER_PROGRAM_LOADED;
+			}
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
+			if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+			{
+				next_state = IANJUTA_DEBUGGER_STARTED;
+			}
+			else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
+			{
+				next_state = IANJUTA_DEBUGGER_PROGRAM_LOADED;
+			}
+			break;
+		case IANJUTA_DEBUGGER_BUSY:
+			return;
+		}
+		dma_queue_emit_debugger_state_change (self, next_state, NULL);
+	}
+}
+
 static void
 dma_queue_emit_debugger_ready (DmaDebuggerQueue *self)
 {
@@ -394,7 +400,7 @@ dma_debugger_queue_complete (DmaDebuggerQueue *self, IAnjutaDebuggerState state)
 static void
 dma_debugger_queue_execute (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT("debugger_queue_execute");
+	DEBUG_PRINT("%s", "debugger_queue_execute");
 
 	/* Check if debugger is connected to a debugger backend */
 	g_return_if_fail (self->debugger != NULL);
@@ -462,9 +468,9 @@ dma_queue_check_state (DmaDebuggerQueue *self, DmaQueueCommand* cmd)
 	{
 		IAnjutaDebuggerState state;
 		
-		if (self->prepend_command)
+		if (self->prepend_command || dma_command_has_flag (cmd, HIGH_PRIORITY))
 		{
-			/* Prepend command use debugger state or current command state */
+			/* Prepend command and high priority command use debugger state or current command state */
 			if (self->last != NULL)
 			{
 				state = dma_command_is_going_to_state (self->last);
@@ -534,7 +540,7 @@ dma_debugger_activate_plugin (DmaDebuggerQueue* self, const gchar *mime_type)
 	{
 		/* No plugin found */
 		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (self->plugin)->shell),
-				_("Unable to find one debugger plugin acception %s mime type"), mime_type);
+				_("Unable to find one debugger plugin supporting a target with %s mime type"), mime_type);
 			
 		return FALSE;
 	}
@@ -563,13 +569,17 @@ dma_debugger_activate_plugin (DmaDebuggerQueue* self, const gchar *mime_type)
 		self->debugger = (IAnjutaDebugger *)anjuta_plugin_manager_get_plugin_by_id (plugin_manager, value);
 
 		self->support = 0;
-		/* Check if cpu interface is available */
-		self->support |= IANJUTA_IS_CPU_DEBUGGER(self->debugger) ? HAS_CPU : 0;
+		/* Check if register interface is available */
+		self->support |= IANJUTA_IS_DEBUGGER_REGISTER(self->debugger) ? HAS_REGISTER : 0;
+		/* Check if memory interface is available */
+		self->support |= IANJUTA_IS_DEBUGGER_MEMORY(self->debugger) ? HAS_MEMORY : 0;
+		/* Check if instruction interface is available */
+		self->support |= IANJUTA_IS_DEBUGGER_INSTRUCTION(self->debugger) ? HAS_INSTRUCTION : 0;
 		/* Check if breakpoint interface is available */
 		self->support |= IANJUTA_IS_DEBUGGER_BREAKPOINT(self->debugger) ? HAS_BREAKPOINT : 0;
 		if (IANJUTA_IS_DEBUGGER_BREAKPOINT (self->debugger))
 		{
-			self->support |= ianjuta_debugger_breakpoint_implement (IANJUTA_DEBUGGER_BREAKPOINT (self->debugger), NULL) * HAS_BREAKPOINT * 2;
+			self->support |= ianjuta_debugger_breakpoint_implement_breakpoint (IANJUTA_DEBUGGER_BREAKPOINT (self->debugger), NULL) * HAS_BREAKPOINT * 2;
 		}			
 		/* Check if variable interface is available */
 		self->support |= IANJUTA_IS_DEBUGGER_VARIABLE(self->debugger) ? HAS_VARIABLE : 0;
@@ -600,7 +610,7 @@ on_dma_debugger_ready (DmaDebuggerQueue *self, IAnjutaDebuggerState state)
 static void
 on_dma_debugger_started (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: receive debugger started");
+	DEBUG_PRINT ("%s", "From debugger: receive debugger started");
 	dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
 }
 
@@ -620,35 +630,35 @@ on_dma_debugger_stopped (DmaDebuggerQueue *self, GError *err)
 static void
 on_dma_program_loaded (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: receive program loaded");
+	DEBUG_PRINT ("%s", "From debugger: receive program loaded");
 	dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
 }
 
 static void
 on_dma_program_running (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: debugger_program_running");
+	DEBUG_PRINT ("%s", "From debugger: debugger_program_running");
 	dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_RUNNING, NULL);
 }
 
 static void
 on_dma_program_stopped (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: receive program stopped");
+	DEBUG_PRINT ("%s", "From debugger: receive program stopped");
 	dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_STOPPED, NULL);
 }
 
 static void
 on_dma_program_exited (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: receive program exited");
+	DEBUG_PRINT ("%s", "From debugger: receive program exited");
 	dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
 }
 
 static void
-on_dma_program_moved (DmaDebuggerQueue *self, guint pid, gint tid, guint address, const gchar* src_path, guint line)
+on_dma_program_moved (DmaDebuggerQueue *self, guint pid, gint tid, gulong address, const gchar* src_path, guint line)
 {
-	DEBUG_PRINT ("From debugger: program moved");
+	DEBUG_PRINT ("%s", "From debugger: program moved");
 	self->prepend_command++;
 	g_signal_emit_by_name (self->plugin, "program-moved", pid, tid, address, src_path, line);
 	self->prepend_command--;
@@ -657,7 +667,7 @@ on_dma_program_moved (DmaDebuggerQueue *self, guint pid, gint tid, guint address
 static void
 on_dma_frame_changed (DmaDebuggerQueue *self, guint frame, gint thread)
 {
-	DEBUG_PRINT ("From debugger: frame changed");
+	DEBUG_PRINT ("%s", "From debugger: frame changed");
 	self->prepend_command++;
 	g_signal_emit_by_name (self->plugin, "frame-changed", frame, thread);
 	self->prepend_command--;
@@ -666,7 +676,7 @@ on_dma_frame_changed (DmaDebuggerQueue *self, guint frame, gint thread)
 static void
 on_dma_signal_received (DmaDebuggerQueue *self, const gchar* name, const gchar* description)
 {
-	DEBUG_PRINT ("From debugger: signal received");
+	DEBUG_PRINT ("%s", "From debugger: signal received");
 	self->prepend_command++;
 	g_signal_emit_by_name (self->plugin, "signal-received", name, description);
 	self->prepend_command--;
@@ -675,7 +685,7 @@ on_dma_signal_received (DmaDebuggerQueue *self, const gchar* name, const gchar*
 static void
 on_dma_sharedlib_event (DmaDebuggerQueue *self)
 {
-	DEBUG_PRINT ("From debugger: shared lib event");
+	DEBUG_PRINT ("%s", "From debugger: shared lib event");
 	self->stop_on_sharedlib = TRUE;
 	dma_debugger_queue_complete (self, IANJUTA_DEBUGGER_PROGRAM_STOPPED);
 	self->prepend_command++;
@@ -696,8 +706,11 @@ dma_debugger_queue_command_callback (const gpointer data, gpointer user_data, GE
 	g_return_if_fail (self->last != NULL);
 	
 	self->prepend_command++;
-	dma_command_callback (self->last, data, err);
-	self->prepend_command--;	
+	if (self->queue_state != IANJUTA_DEBUGGER_STOPPED)
+	{
+		dma_command_callback (self->last, data, err);
+	}
+	self->prepend_command--;
 }
 
 gboolean
@@ -724,9 +737,29 @@ dma_debugger_queue_append (DmaDebuggerQueue *self, DmaQueueCommand *cmd)
 			/* Append command at the beginning */
 			g_queue_push_head (self->queue, cmd);
 			
-			dma_debugger_queue_complete (self, self->debugger_state);
+			/* We must not interrupt command having callback, as the command
+			 * will be removed, the callback when emitted will be redirected to
+			 * the handler of the next command */
+			if ((state == IANJUTA_DEBUGGER_STOPPED) || (self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING))
+			{
+				dma_debugger_queue_complete (self, self->debugger_state);
+			}
 		}
-		else if (self->prepend_command == 0)
+		else if ((self->prepend_command > 0) || dma_command_has_flag (cmd, HIGH_PRIORITY))
+		{
+			IAnjutaDebuggerState state;
+			
+			state = dma_command_is_going_to_state (cmd);
+			if (state != IANJUTA_DEBUGGER_BUSY)
+			{
+				/* Command is changing debugger state */
+				dma_queue_cancel_unexpected (self, state);
+			}
+			
+			/* Prepend command at the beginning */
+			g_queue_push_head (self->queue, cmd);
+		}
+		else
 		{
 			/* Append command at the end (in the queue) */
 			IAnjutaDebuggerState state;
@@ -739,11 +772,6 @@ dma_debugger_queue_append (DmaDebuggerQueue *self, DmaQueueCommand *cmd)
 				self->queue_state = state;
 			}
 		}
-		else
-		{
-			/* Prepend command at the beginning */
-			g_queue_push_head (self->queue, cmd);
-		}
 	
 		dma_debugger_queue_execute(self);
 		
@@ -868,7 +896,7 @@ dma_debugger_queue_dispose (GObject *obj)
 
 	dma_debugger_queue_clear (self);
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 /* finalize is the last destruction step. It must free all memory allocated
@@ -878,9 +906,11 @@ dma_debugger_queue_dispose (GObject *obj)
 static void
 dma_debugger_queue_finalize (GObject *obj)
 {
-	/*DmaDebuggerQueue *self = DMA_DEBUGGER_QUEUE (obj);*/
+	DmaDebuggerQueue *self = DMA_DEBUGGER_QUEUE (obj);
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	g_queue_free (self->queue);
+
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 /* instance_init is the constructor. All functions should work after this
