@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2002, 2003  Sebastian Rittau <srittau@jroger.in-berlin.de>
- * $Id: egg-datetime.c 3310 2007-11-18 12:45:16Z jhs $
+ * $Id: egg-datetime.c 4655 2009-01-31 16:30:54Z sgranjoux $
  *
  * Based on GnomeDateEdit by Miguel de Icaza.
  *
@@ -249,10 +249,10 @@ static void	 update_time_label		(EggDateTime *edt);
 static GtkHBoxClass *parent_class = NULL;
 
 
-	GtkType
+GType
 egg_datetime_get_type (void)
 {
-	static GtkType datetime_type = 0;
+	static GType datetime_type = 0;
 
 	if (!datetime_type) {
 		static const GTypeInfo datetime_info = {
@@ -605,7 +605,7 @@ egg_datetime_set_property (GObject	*object,
 			priv->year = g_value_get_uint (value);
 			break;
 		case ARG_MONTH:
-			priv->minute = g_value_get_uint (value);
+			priv->month = g_value_get_uint (value);
 			break;
 		case ARG_DAY:
 			priv->day = g_value_get_uint (value);
@@ -932,7 +932,7 @@ cal_set_today (EggDateTime *edt, GtkCalendar *calendar)
 	edt->priv->no_date = FALSE;
 
 	now = g_date_new();
-	g_date_set_time (now, time (NULL));
+	g_date_set_time_t (now, time (NULL));
 
 	year = g_date_get_year(now);
 	month = g_date_get_month(now);
