@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
  
 #ifndef MESSAGE_VIEW_H
@@ -63,6 +63,7 @@ GtkWidget* message_view_new (AnjutaPreferences* prefs, GtkWidget* popup_menu);
 void message_view_next(MessageView* view);
 void message_view_previous(MessageView* view);
 void message_view_save(MessageView* view);
+void message_view_copy(MessageView* view);
 gboolean message_view_serialize (MessageView *view,
 								 AnjutaSerializer *serializer);
 gboolean message_view_deserialize (MessageView *view,
