@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: anjuta-view.c 4055 2008-07-05 15:08:05Z jhs $ 
+ * $Id: anjuta-view.c 4496 2008-12-28 09:57:13Z jhs $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -66,7 +66,7 @@ struct _AnjutaViewPrivate
   Sourceview* sv;
 };
 
-static void	anjuta_view_destroy		(GtkObject       *object);
+static void	anjuta_view_dispose		(GObject       *object);
 static void	anjuta_view_finalize		(GObject         *object);
 static void	anjuta_view_move_cursor		(GtkTextView     *text_view,
 						 GtkMovementStep  step,
@@ -162,13 +162,12 @@ static void
 anjuta_view_class_init (AnjutaViewClass *klass)
 {
 	GObjectClass     *object_class = G_OBJECT_CLASS (klass);
-	GtkObjectClass   *gtkobject_class = GTK_OBJECT_CLASS (klass);
 	GtkTextViewClass *textview_class = GTK_TEXT_VIEW_CLASS (klass);
 	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkBindingSet    *binding_set;
 	GParamSpec *anjuta_view_spec_popup;
 
-	gtkobject_class->destroy = anjuta_view_destroy;
+	object_class->dispose = anjuta_view_dispose;
 	object_class->finalize = anjuta_view_finalize;
 	object_class->set_property = anjuta_view_set_property;
 	object_class->get_property = anjuta_view_get_property;	
@@ -312,13 +311,16 @@ anjuta_view_init (AnjutaView *view)
 }
 
 static void
-anjuta_view_destroy (GtkObject *object)
+anjuta_view_dispose (GObject *object)
 {
 	AnjutaView *view;
 
 	view = ANJUTA_VIEW (object);
 	
-	(* GTK_OBJECT_CLASS (anjuta_view_parent_class)->destroy) (object);
+	if (view->priv->scroll_idle > 0)
+				g_source_remove (view->priv->scroll_idle);
+		
+	(* G_OBJECT_CLASS (anjuta_view_parent_class)->dispose) (object);
 }
 
 static void
