@@ -170,7 +170,9 @@ static void sourceview_create_highligth_indic(Sourceview* sv)
 	sv->priv->warning_indic = 
 		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
 									WARNING_INDIC,
-									"foreground", "#00FF00", NULL); 
+									"foreground", "#00FF00", 
+									"underline", PANGO_UNDERLINE_SINGLE,
+									NULL); 
 	sv->priv->critical_indic = 
 		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
 									CRITICAL_INDIC,
@@ -190,16 +192,45 @@ goto_line (Sourceview* sv, gint line)
 	gtk_text_buffer_select_range (buffer, &iter, &iter);
 }
 
+static void
+on_destroy_message_area (Sourceview* sv, GObject *finalized_object)
+{
+	sv->priv->message_area = NULL;
+	g_signal_emit_by_name (G_OBJECT (sv), "update-save-ui");
+}
+
+static void
+sourceview_set_message_area (Sourceview* sv,  GtkWidget *message_area)
+{
+	if (sv->priv->message_area != NULL)
+		gtk_widget_destroy (sv->priv->message_area);
+	sv->priv->message_area = message_area;
+
+	if (sv->priv->message_area == NULL)
+		return;
+		
+	gtk_widget_show (message_area);
+	gtk_box_pack_start (GTK_BOX (sv),
+						message_area,
+						FALSE,
+						FALSE,
+						0);
+	g_object_weak_ref (G_OBJECT (sv->priv->message_area),
+					   (GWeakNotify)on_destroy_message_area, sv);
+	
+	g_signal_emit_by_name (G_OBJECT (sv), "update-save-ui");
+}
+
 /* Callbacks */
 
 static void
-on_assist_window_destroyed (AssistWindow* window, Sourceview* sv)
+on_assist_window_destroyed (Sourceview* sv, gpointer where_object_was)
 {
 	sv->priv->assist_win = NULL;
 }
 
 static void 
-on_assist_tip_destroyed (AssistTip* tip, Sourceview* sv)
+on_assist_tip_destroyed (Sourceview* sv, gpointer where_object_was)
 {
 	sv->priv->assist_tip = NULL;
 }
@@ -249,7 +280,7 @@ static void on_insert_text (GtkTextBuffer* buffer,
 static void on_document_modified_changed(GtkTextBuffer* buffer, Sourceview* sv)
 {
 	/* Emit IAnjutaFileSavable signals */
-	g_signal_emit_by_name(G_OBJECT(sv), "save_point",
+	g_signal_emit_by_name(G_OBJECT(sv), "update-save-ui",
 						  !gtk_text_buffer_get_modified(buffer));
 }
 
@@ -299,13 +330,31 @@ on_reload_dialog_response (GtkWidget *message_area, gint res, Sourceview *sv)
 	gtk_widget_destroy (message_area);
 }
 
+static void 
+on_close_dialog_response (GtkWidget *message_area, gint res, Sourceview *sv)
+{
+	if (res == GTK_RESPONSE_YES)
+	{
+		IAnjutaDocumentManager *docman;
+		
+		docman = anjuta_shell_get_interface (sv->priv->plugin->shell, IAnjutaDocumentManager, NULL);
+		if (docman == NULL) return;
+
+		ianjuta_document_manager_remove_document (docman, IANJUTA_DOCUMENT (sv), FALSE, NULL);
+	}
+	else
+	{
+		/* Set dirty */
+		gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(sv->priv->document), TRUE);
+	}
+	gtk_widget_destroy (message_area);
+}
+
 static gboolean
 on_file_changed (SourceviewIO* sio, Sourceview* sv)
 {
 	GtkWidget *message_area;
-	IAnjutaDocumentManager *docman;
 	IAnjutaDocument *doc;
-	AnjutaShell *shell;
 	gchar *buff;
 	
 	gchar* filename = sourceview_io_get_filename (sio);
@@ -318,10 +367,6 @@ on_file_changed (SourceviewIO* sio, Sourceview* sv)
 
 	g_free (filename);
 	
-	shell = ANJUTA_PLUGIN (sv->priv->plugin)->shell;
-	docman = anjuta_shell_get_interface (shell, IAnjutaDocumentManager, NULL);
-	if (!docman)
-  		return TRUE;
   	doc = IANJUTA_DOCUMENT (sv);
 
 	message_area = anjuta_message_area_new (buff, GTK_STOCK_DIALOG_WARNING);
@@ -335,8 +380,46 @@ on_file_changed (SourceviewIO* sio, Sourceview* sv)
 	
 	g_signal_connect (G_OBJECT(message_area), "response",
 					  G_CALLBACK (on_reload_dialog_response),
-					  sv);					  
-	ianjuta_document_manager_set_message_area (docman, doc, message_area, NULL);
+					  sv);
+	
+	sourceview_set_message_area (sv, message_area);
+	
+	return FALSE;
+}
+
+static gboolean
+on_file_deleted (SourceviewIO* sio, Sourceview* sv)
+{
+	GtkWidget *message_area;
+	IAnjutaDocument *doc;
+	gchar *buff;
+	
+	gchar* filename = sourceview_io_get_filename (sio);
+	
+	buff =
+		g_strdup_printf (_
+						 ("The file '%s' has been deleted on the disk.\n"
+						  "Do you want to close it?"),
+						 filename);
+
+	g_free (filename);
+	
+  	doc = IANJUTA_DOCUMENT (sv);
+
+	message_area = anjuta_message_area_new (buff, GTK_STOCK_DIALOG_WARNING);
+	anjuta_message_area_add_button (ANJUTA_MESSAGE_AREA (message_area),
+									GTK_STOCK_DELETE,
+									GTK_RESPONSE_YES);
+	anjuta_message_area_add_button (ANJUTA_MESSAGE_AREA (message_area),
+								    GTK_STOCK_CANCEL,
+									GTK_RESPONSE_NO);
+	g_free (buff);
+	
+	g_signal_connect (G_OBJECT(message_area), "response",
+					  G_CALLBACK (on_close_dialog_response),
+					  sv);
+	
+	sourceview_set_message_area (sv, message_area);
 	
 	return FALSE;
 }
@@ -364,8 +447,7 @@ on_open_failed (SourceviewIO* io, GError* err, Sourceview* sv)
 										GTK_RESPONSE_OK);
 		g_signal_connect (message_area, "response", G_CALLBACK(gtk_widget_destroy), NULL);
 		
-		ianjuta_document_manager_set_message_area (docman, IANJUTA_DOCUMENT(sv), 
-												   message_area, NULL);
+		sourceview_set_message_area (sv, message_area);
 	}
 	else
 	{
@@ -392,7 +474,6 @@ on_read_only_dialog_response (GtkWidget *message_area, gint res, Sourceview *sv)
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (sv->priv->view),
 									TRUE);
 		sv->priv->read_only = FALSE;
-		g_signal_emit_by_name (sv, "save-point", TRUE);
 	}
 	gtk_widget_destroy (message_area);
 }
@@ -407,13 +488,8 @@ on_open_finish(SourceviewIO* io, Sourceview* sv)
 	
 	if (sourceview_io_get_read_only (io))
 	{
-		AnjutaShell* shell = ANJUTA_PLUGIN (sv->priv->plugin)->shell;
-		IAnjutaDocumentManager* docman = 
-			anjuta_shell_get_interface (shell, IAnjutaDocumentManager, NULL);
-		g_return_if_fail (docman != NULL);
-		IAnjutaDocument* doc = IANJUTA_DOCUMENT (sv);
 		gchar* filename = sourceview_io_get_filename (io);
-		gchar* buff = g_strdup_printf ("The file '%s' is read-only! Edit anyway?",
+		gchar* buff = g_strdup_printf (_("The file '%s' is read-only! Edit anyway?"),
 									   filename);
 		GtkWidget* message_area;
 		g_free (filename);
@@ -433,13 +509,12 @@ on_open_finish(SourceviewIO* io, Sourceview* sv)
 		
 		sv->priv->read_only = TRUE;
 		
-		ianjuta_document_manager_set_message_area (docman, doc, message_area, NULL);
+		sourceview_set_message_area (sv, message_area);
 	}
 	else
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (sv->priv->view), TRUE);
 
-    g_signal_emit_by_name(G_OBJECT(sv), "save_point",
-						  TRUE);
+    g_signal_emit_by_name(G_OBJECT(sv), "update-save-ui");
 	
 	if (sv->priv->goto_line > 0)
 	{
@@ -470,6 +545,8 @@ static void on_save_failed (SourceviewIO* sio, GError* err, Sourceview* sv)
 	GList* documents = ianjuta_document_manager_get_doc_widgets (docman, NULL);
 	GtkWidget* message_area;
 	
+	g_signal_emit_by_name(G_OBJECT(sv), "saved", NULL);
+	
 	/* Could not open <filename>: <error message> */
 	gchar* message = g_strdup_printf (_("Could not save %s: %s"),
 									  sourceview_io_get_filename (sv->priv->io), 
@@ -483,8 +560,7 @@ static void on_save_failed (SourceviewIO* sio, GError* err, Sourceview* sv)
 										GTK_RESPONSE_OK);
 		g_signal_connect (message_area, "response", G_CALLBACK(gtk_widget_destroy), NULL);
 		
-		ianjuta_document_manager_set_message_area (docman, IANJUTA_DOCUMENT(sv), 
-												   message_area, NULL);
+		sourceview_set_message_area (sv, message_area);
 	}
 	else
 	{
@@ -508,7 +584,7 @@ static void on_save_finish(SourceviewIO* sio, Sourceview* sv)
 	gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(sv->priv->document), FALSE);
 	sv->priv->read_only = FALSE;
 	g_signal_emit_by_name(G_OBJECT(sv), "saved", file);
-	g_signal_emit_by_name(G_OBJECT(sv), "save_point", TRUE);
+	g_signal_emit_by_name(G_OBJECT(sv), "update-save-ui");
 	g_object_unref (file);
 	/* Autodetect language */
 	ianjuta_editor_language_set_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL, NULL);
@@ -537,6 +613,7 @@ sourceview_instance_init(Sourceview* sv)
 	sv->priv = g_slice_new0 (SourceviewPrivate);
 	sv->priv->io = sourceview_io_new (sv);
 	g_signal_connect (sv->priv->io, "changed", G_CALLBACK (on_file_changed), sv);
+	g_signal_connect (sv->priv->io, "deleted", G_CALLBACK (on_file_deleted), sv);
 	g_signal_connect (sv->priv->io, "open-finished", G_CALLBACK (on_open_finish),
 					  sv);
 	g_signal_connect (sv->priv->io, "open-failed", G_CALLBACK (on_open_failed),
@@ -640,12 +717,15 @@ sourceview_new(GFile* file, const gchar* filename, AnjutaPlugin* plugin)
 	sv->priv->plugin = plugin;
 	
 	/* Add View */
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv),
+	sv->priv->window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_box_pack_end (GTK_BOX (sv), sv->priv->window, TRUE, TRUE, 0);
+	
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv->priv->window),
 				      GTK_POLICY_AUTOMATIC,
 				      GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(sv), GTK_WIDGET(sv->priv->view));
+	gtk_container_add(GTK_CONTAINER(sv->priv->window), GTK_WIDGET(sv->priv->view));
 	gtk_widget_show_all(GTK_WIDGET(sv));
-	v_adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sv));
+	v_adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sv->priv->window));
 	g_signal_connect (v_adj, "value-changed", G_CALLBACK (sourceview_adjustment_changed), sv);
 	
 	if (file != NULL)
@@ -733,6 +813,13 @@ ifile_savable_is_read_only (IAnjutaFileSavable* file, GError** e)
 	return sv->priv->read_only;
 }
 
+static gboolean
+ifile_savable_is_conflict (IAnjutaFileSavable* file, GError** e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW (file);
+	return sv->priv->message_area != NULL;
+}
+
 static void
 isavable_iface_init (IAnjutaFileSavableIface *iface)
 {
@@ -741,6 +828,7 @@ isavable_iface_init (IAnjutaFileSavableIface *iface)
 	iface->set_dirty = ifile_savable_set_dirty;
 	iface->is_dirty = ifile_savable_is_dirty;
 	iface->is_read_only = ifile_savable_is_read_only;
+	iface->is_conflict = ifile_savable_is_conflict;
 }
 
 static void
@@ -895,7 +983,7 @@ static gint ieditor_get_lineno(IAnjutaEditor *editor, GError **e)
 static gint ieditor_get_length(IAnjutaEditor *editor, GError **e)
 {
 	/* We do not use gtk_text_buffer_get_char_count here because
-	someone may rely that this is the size of a buffer for all the text */
+	 we need a size in bytes not in characters */
 	
 	GtkTextIter start_iter;
 	GtkTextIter end_iter;
@@ -909,7 +997,7 @@ static gint ieditor_get_length(IAnjutaEditor *editor, GError **e)
 								   &end_iter);
 	text = gtk_text_buffer_get_slice(GTK_TEXT_BUFFER(sv->priv->document),
 									&start_iter, &end_iter, TRUE);
-	length = g_utf8_strlen(text,  -1);
+	length = strlen(text);
 	g_free(text);
 
 	return length;
@@ -1312,27 +1400,22 @@ idocument_iface_init (IAnjutaDocumentIface *iface)
 }
 
 static void
-set_select(Sourceview* sv, GtkTextIter* start_iter, GtkTextIter* end_iter)
+set_select(Sourceview* sv, GtkTextIter* start_iter, GtkTextIter* end_iter, gboolean scroll)
 {
 	GtkTextBuffer* buffer = GTK_TEXT_BUFFER (sv->priv->document);
 	gtk_text_buffer_select_range (buffer, start_iter, end_iter);
-	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (sv->priv->view),
-								  gtk_text_buffer_get_insert (buffer),
-								  0.25,
-								  FALSE,
-								  0.0,
-								  0.0);
+
+	if (scroll)
+		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (sv->priv->view),
+									  gtk_text_buffer_get_insert (buffer),
+									  0.25,
+									  FALSE,
+									  0.0,
+									  0.0);
 }
 
 /* IAnjutaEditorSelection */
 
-// TODO: Move these to ilanguage-support?
-static void
-iselect_to_brace(IAnjutaEditorSelection* edit, GError** e)
-{
-	
-}
-
 /* Find the previous open brace that begins the current indentation level. */
 static gboolean find_open_bracket(GtkTextIter *iter)
 {
@@ -1366,7 +1449,7 @@ static gboolean find_close_bracket(GtkTextIter *iter)
 		{
 		case '{':
 			++level;
-			break;
+			break;				
 		case '}':
 			if (!--level)
 				return TRUE;
@@ -1395,7 +1478,7 @@ iselect_block(IAnjutaEditorSelection* edit, GError** e)
 		if (find_close_bracket (&end_iter))
 		{
 			gtk_text_iter_forward_char (&end_iter);	  /* move past brace */
-			set_select (sv, &iter, &end_iter);
+			set_select (sv, &iter, &end_iter, TRUE);
 		}
 	}
 }
@@ -1404,12 +1487,14 @@ static void
 iselect_set (IAnjutaEditorSelection* edit, 
 			 IAnjutaIterable* istart,
 			 IAnjutaIterable* iend,
+			 gboolean scroll,
 			 GError** e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
 	set_select(sv,
 			   sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
-			   sourceview_cell_get_iter (SOURCEVIEW_CELL (iend)));
+			   sourceview_cell_get_iter (SOURCEVIEW_CELL (iend)),
+			   scroll);
 }
 															
 
@@ -1512,7 +1597,6 @@ iselect_iface_init(IAnjutaEditorSelectionIface *iface)
 	iface->select_block = iselect_block;
 	iface->select_function = iselect_function;
 	iface->select_all = iselect_all;
-	iface->select_to_brace = iselect_to_brace;
 	iface->select_block = iselect_block;
 	iface->get = iselect_get;
 	iface->replace = iselect_replace;
@@ -1803,15 +1887,17 @@ ilanguage_get_supported_languages (IAnjutaEditorLanguage *ilanguage,
 	static GList* languages = NULL;
 	if (!languages)
 	{
-		GStrv langs;
-		GStrv lang;
-		g_object_get (gtk_source_language_manager_get_default(), "language-ids", &langs, NULL);
-		
-		for (lang = langs; *lang != NULL; lang++)
+		const gchar* const * langs = gtk_source_language_manager_get_language_ids (gtk_source_language_manager_get_default());		
+		if (langs)
 		{
-			languages = g_list_append (languages, *lang);
+			const gchar* const * lang;
+		
+			for (lang = langs; *lang != NULL; lang++)
+			{
+				languages = g_list_append (languages, (gpointer)*lang);
+			}
 		}
-	}		
+	}
 	return languages;
 }
 
@@ -1853,15 +1939,13 @@ ilanguage_set_language (IAnjutaEditorLanguage *ilanguage,
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW (ilanguage);
 	gboolean found = FALSE;
-	GStrv languages;
-	GStrv cur_lang;
-	g_object_get (G_OBJECT (gtk_source_language_manager_get_default ()), "language-ids",
-							&languages, NULL);
-	for (cur_lang = languages; *cur_lang != NULL && language != NULL; cur_lang++)
+	const GList* languages = ilanguage_get_supported_languages(ilanguage, err);
+	const GList* cur_lang;
+	for (cur_lang = languages; cur_lang != NULL && language != NULL; cur_lang = g_list_next (cur_lang))
 	{
 		GtkSourceLanguage* source_language = 
 			gtk_source_language_manager_get_language (gtk_source_language_manager_get_default(),
-													  *cur_lang);
+													  cur_lang->data);
 		const gchar* id = gtk_source_language_get_id (source_language);
 		
 		if (g_str_equal (language, id))
@@ -1874,7 +1958,6 @@ ilanguage_set_language (IAnjutaEditorLanguage *ilanguage,
 			break;
 		}
 	}
-	g_strfreev(languages);
 	if (!found)
 	{
 		autodetect_language (sv);
@@ -1962,8 +2045,8 @@ iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices, IAnjutaIterable*
 		{
 			sv->priv->assist_win = assist_window_new(GTK_TEXT_VIEW(sv->priv->view), NULL,
 													 ianjuta_iterable_get_position (ipos, NULL));
-			g_signal_connect(G_OBJECT(sv->priv->assist_win), "destroy", 
-								 G_CALLBACK(on_assist_window_destroyed), sv);
+			g_object_weak_ref (G_OBJECT(sv->priv->assist_win),
+			                   (GWeakNotify)on_assist_window_destroyed, sv);
 			g_signal_connect(G_OBJECT(sv->priv->assist_win), "chosen", 
 								 G_CALLBACK(on_assist_chosen), sv);
 			g_signal_connect(G_OBJECT(sv->priv->assist_win), "cancel", 
@@ -2017,8 +2100,9 @@ iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips, IAnjutaIterable* i
 		sv->priv->assist_tip = 
 			ASSIST_TIP (assist_tip_new (GTK_TEXT_VIEW (sv->priv->view), tips));
 		
-		g_signal_connect (G_OBJECT(sv->priv->assist_tip), "destroy", G_CALLBACK(on_assist_tip_destroyed),
-						  sv);
+		g_object_weak_ref (G_OBJECT(sv->priv->assist_tip),
+		                   (GWeakNotify) on_assist_tip_destroyed,
+		                   sv);
 		assist_tip_move (sv->priv->assist_tip, GTK_TEXT_VIEW (sv->priv->view), tip_position);
 		gtk_widget_show (GTK_WIDGET (sv->priv->assist_tip));
 	}
@@ -2214,7 +2298,7 @@ ihover_iface_init(IAnjutaEditorHoverIface* iface)
 	iface->display = ihover_display;
 }
 
-ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_SCROLLED_WINDOW);
+ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_VBOX);
 ANJUTA_TYPE_ADD_INTERFACE(idocument, IANJUTA_TYPE_DOCUMENT);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
 ANJUTA_TYPE_ADD_INTERFACE(isavable, IANJUTA_TYPE_FILE_SAVABLE);
